﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Chess.Test
{
    [TestClass]
    public class BoardTest
    {
        private Pawn Piece { get; set; }
        private Board Target { get; set; }

        [TestInitialize]
        public void BeforeEachTest()
        {
            Piece = new Pawn();
            Target = new Board();
        }

        [TestClass]
        public class GetPiece : BoardTest
        {
            [TestMethod]
            public void RetrievesPieceAddedToLocation()
            {
                var bc = new BoardCoordinate(1,1);
                Target.AddPiece(Piece, bc);
                Assert.AreEqual(Piece, Target.GetPiece(bc));
            }
        }

        [TestClass]
        public class AddPiece : BoardTest
        {
            [TestMethod]
            public void DoesNotThrowExceptionWhenAddingAPieceToAnUnoccupiedSquare()
            {
                Target.AddPiece(new Pawn(), new BoardCoordinate(1,2));
            }

            [TestMethod]
            public void ThrowsExceptionWhenBoardCoordinateXIsLargerThanBoardSize()
            {
                var coordinate = new BoardCoordinate(9,1);
                ExtendedAssert.Throws<ArgumentOutOfRangeException>(() => Target.AddPiece(Piece, coordinate));
            }

            [TestMethod]
            public void ThrowsExceptionWhenBoardCoordinateYIsLargerThanBoardSize()
            {
                var coordinate = new BoardCoordinate(1, 9);
                ExtendedAssert.Throws<ArgumentOutOfRangeException>(() => Target.AddPiece(Piece, coordinate));
            }

            [TestMethod]
            public void ThrowsExceptionWhenBoardCoordinateXIsZero()
            {
                var coordinate = new BoardCoordinate(0, 1);
                ExtendedAssert.Throws<ArgumentOutOfRangeException>(() => Target.AddPiece(Piece, coordinate));
            }

            [TestMethod]
            public void ThrowsExceptionWhenBoardCoordinateYIsZero()
            {
                var coordinate = new BoardCoordinate(1, 0);
                ExtendedAssert.Throws<ArgumentOutOfRangeException>(() => Target.AddPiece(Piece, coordinate));
            }
        }
    }
}
